/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: cs-list-ep.h,v 1.8 2003/03/23 18:50:38 vjt Exp $ */

#ifndef __cs_list_ep_h__
#define __cs_list_ep_h__

#include "parse.h"
#include "cs-list.h"

/* parv[0] -> channel name
 * parv[1] -> string to parse
 */
#define LIST_ENTRY_POINT(NAME, STR_NAME, USER_PTR, HASH_FUNC, LIST_ID)\
	void (NAME)(User *caller, gint parc, gchar **parv)	\
	{							\
	    const struct STR_NAME *c;				\
	    extern User * USER_PTR;				\
	    register gchar *s;					\
	    gint len = 0;					\
	    gchar *str, *arg = NULL;				\
	    							\
	    if(parv && parv[0] && parv[1])			\
	    {							\
		str = parv[1];					\
		s = strchr(str, ' ');				\
		if(s)						\
		{						\
		    *s = '\0';					\
		    len = s++ - str;				\
		}						\
		else if(str)					\
		{						\
		    len = strlen(str);				\
		}						\
								\
		my_strtoupper(str);				\
								\
		if((c = HASH_FUNC (str, len)))			\
		{						\
		    if(c->para && s)				\
		    {						\
			/* command with params			\
			 */					\
			arg = s;				\
		    }						\
		    (*c->func) (caller, LIST_ID, parv[0], arg);	\
								\
		    return;					\
		}						\
	    }							\
	    else						\
	    {							\
		send_out(":%s NOTICE %s :Not enough parameters,"\
			" type /MSG %s HELP %s for more "	\
			"information", (USER_PTR)->name,	\
			caller->name, (USER_PTR)->name,		\
			cs_lists[LIST_ID].name);		\
		return;						\
								\
	    }							\
	    							\
	    /* unknown command ..				\
	     */							\
	    send_out(":%s NOTICE %s :Unknown command (%s), "	\
		    "type /MSG %s HELP %s for a list "	\
		    "of available commands",			\
		    (USER_PTR)->name, caller->name, parv[1],	\
		    (USER_PTR)->name, cs_lists[LIST_ID].name);	\
	    return;						\
	}

#endif
