/*
 * Copyright 2002 Barnaba Marcello <vjt@azzurra.org>
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2a. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 2b. Redistribution in binary form requires specific prior written
 *     authorization of the maintainer.
 * 
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Barnaba Marcello.
 * 
 * 4. The names of the maintainer, developers and contributors may not be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE MAINTAINER, DEVELOPERS AND
 * CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE DEVELOPERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/* $Id: modules.h,v 1.12 2003/03/27 01:30:37 vjt Exp $ */

#ifndef __modules_h__
#define __modules_h__

#include "sux.h"
#include "agents.h"

#define CHANSERV_MODULE_ID	0
#define NICKSERV_MODULE_ID	1
#define MEMOSERV_MODULE_ID	2
#define OPERSERV_MODULE_ID	3
#define ROOTSERV_MODULE_ID	4

typedef struct module_description ModDesc;
struct module_description
{
    gchar *name;
    gchar *version;
    gchar *info;
    GModule *mod;
};

void load_modules(void);
ModDesc *module_load(gchar *);
gboolean module_unload(gchar *);
ModDesc *module_find(gchar *);

/* messages the dispatcher sends to services agents
 */
#define MESSAGE_TOPIC		0
#define MESSAGE_NEWNICK		1
#define MESSAGE_NICKCHANGE	2
#define MESSAGE_NEWCHAN		3
#define MESSAGE_JOIN		4
#define MESSAGE_CSREQ		5
#define MESSAGE_NSREQ		6
#define MESSAGE_MSREQ		7
#define MESSAGE_OSREQ		8
#define MESSAGE_RSREQ		9
#define MESSAGE_IDENTIFY	10
#define MESSAGE_EXPIRE		11

/* dummy
 */
#define MESSAGE_LAST		12

/* general purpouse dispatcher <-> agent communication
 */
typedef gboolean (*message_func)(gpointer, gpointer);

typedef enum
{
    SUX_MSG_PRIORITY_HIGHEST,
    SUX_MSG_PRIORITY_HIGH,
    SUX_MSG_PRIORITY_NORMAL,
    SUX_MSG_PRIORITY_LOW
} SuxMsgPriority;

typedef struct
{
    gint priority;
    message_func callback;
} SuxMessage;

void message_del_callback(gint, message_func);
void message_add_callback(gint, SuxMsgPriority, message_func);
G_INLINE_FUNC void message_dispatch(gint, gpointer, gpointer);

#endif
