/* 
 * taken from bahamut/src/ircsprintf.c.
 *
 * no copyright into the original file, but bahamut is under the GPL2 license.
 * so, i think, also this file should be under that license.
 * author: unknown. if you are the author of the original code, drop me a note
 * at vjt@openssl.it, so i`ll update this comment. thanks.
 *
 * modified a bit:
 * 1) glibized
 * 2) re-indented
 * 
 * -vjt
 */

/* $Id: sux-printf.c,v 1.4 2003/03/27 02:10:30 vjt Exp $ */

#include "sux-printf.h"
#include "log.h"

static gchar num[12] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
};
static gchar itoa_tab[10] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9'
};
static gchar xtoa_tab[16] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'a', 'b', 'c', 'd', 'e', 'f'
};

static gchar *nullstring = "(null)";

G_INLINE_FUNC gint sux_vsnprintf(gchar *buf, gsize size, const gchar *format, va_list ap) 
{
    register gchar *s = NULL;
    gulong i, u;
    gint len = 0;

    gchar *str = buf;
    const gchar *pattern = format;
    va_list vl;

    G_VA_COPY(vl, ap);

    while(*format && len < size)
    {
	u = 0;
	switch(*format)
	{
	    case '%':
		format++;
		switch(*format)
		{
		    case '%':
			buf[len++] = '%';
			format++;
			break;

		    case 's': /* most popular ;) */
			s = va_arg(ap, gchar *);

			if(s == NULL)
			    s = nullstring;
			while(*s && len < size)
			    buf[len++] = *s++;

			format++;
			break;
		    case 'u':
			format--;
			/* falls through and is caught below */
		    case 'l':
			if (*(format+1) == 'u')
			{
			    u = 1;
			    format++;
			}
			else if (*(format+1) == 'd')
			{
			    u = 0;
			    format++;
			}
			else
			    u = 0;
			/* fallthrough */
		    case 'd':
		    case 'i':
			i = va_arg(ap, gulong);
			if(!u)
			{
			    if(i & 0x80000000)
			    {
				buf[len++] = '-'; /* it's negative.. */
				i = 0x80000000 - (i & ~0x80000000);
			    }
			    s = &num[11];

			    do
			    {
				*--s = itoa_tab[i % 10];
				i /= 10;
			    } while(i != 0);

			    while(*s)
			    {
				buf[len++] = *s++;
			    }

			    format++;
			    break;
			}
		    case 'n':
			/* oo, sneaky...it really is just a long, though! */
		    case 'x':
		    case 'X':
			i = va_arg(ap, glong);
			buf[len++] = '0';
			buf[len++] = 'x';
			s = &num[11];

			do
			{
			    *--s = xtoa_tab[i % 16];
			    i /= 16;
			} while(i != 0);

			while(*s)
			{
			    buf[len++]=*s++;
			}

			format++;
			break;

		    case 'c':
			buf[len++] = (gchar) va_arg(ap, gint);
		    format++;
		    break;
		default:
		    /* yick, unknown type...default to returning what our 
		       snprintf friend would */

		    return g_vsnprintf(str, size, pattern, vl);
		}
		break;
	    default:
		buf[len++] = *format++;
		break;
	}
    }

    buf[len] = '\0';
    return len;
}

G_INLINE_FUNC gint sux_snprintf(gchar *str, gsize size, const gchar *format, ...)
{
    gint ret;
    va_list vl;

    va_start(vl, format);
    ret = sux_vsnprintf(str, size, format, vl);
    va_end(vl);

    return ret;
}
